/* $Header: xdefaults.c,v 1.3 88/02/27 16:35:03 rws Exp $ */
/*      This code gets the command line options and deals with them.
 *
 *	Created: 	10/09/87
 *	Last changed: 	10/19/87
 *	By:		Chris Peterson
 *	
 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "xdefs.h"		/* The x specific header file. */

/* This is a rather brute force method of gettin this info, but it
   works so why fix it? */

/*	Function Name: xw_getdefaults
 *	Discription: This function gets the X defaults.
 *	Arguments: xw_defs - a pointer to the Xdefs structure (see xman.h)
 *                 strs,number  - argc and argv
 *                 flag - a flag that is 1 if a position has been specified.
 *	Returns: none - but sets up the xdisplay structure.
 */

void
xw_getdefaults(dpy,xw_defs,strs,number)
Display *dpy;
Xdefs * xw_defs;
char * strs[];
int number;
{

  char *tempstr;
  int i;

  xw_defs->font = FONT;

  tempstr = XGetDefault(dpy,strs[0],"reversevideo");
  if ((tempstr != NULL) && (0 == strcmp("on",tempstr))) {
    xw_defs->bgcolor = FGCOLOR;
    xw_defs->fgcolor = BGCOLOR;
  }
  else {
    xw_defs->fgcolor = FGCOLOR;
    xw_defs->bgcolor = BGCOLOR;
  }

  tempstr = XGetDefault(dpy,strs[0],"font");         /* Load the Font. */
  if (tempstr != NULL)
    xw_defs->font = tempstr;

  tempstr = XGetDefault(dpy,strs[0],"geometry");

/* A Flag that tells us wether or not to place the  window. */

  if (tempstr == NULL)
    xw_defs->flag = 0;  
  else {
    xw_defs->flag = 1;
    get_geom(tempstr,xw_defs);
  }

/* this part looks at the command line options and deals with them. 
   since they have already been checked there is no need to deal with errors.*/
  
  for (i = 1; i < number; i++) {
    if (strs[i][0] == '-') {
      switch (strs[i][1]) {
	case 'r':			/* switch to reverse video. */
	  xw_defs->bgcolor = FGCOLOR;      /* Note how this overrides */ 
	  xw_defs->fgcolor = BGCOLOR; 	/* the .Xdefaults. */
	  continue;
	case 'f':
	  i++;
	  xw_defs->font = strs[i];
	  continue;
	}
    }
    if (strs[i][0] == '=') {
      xw_defs->flag = 1;
      get_geom(strs[i],xw_defs);
      continue;
    }
  }
}

/*	Function Name: usage
 *	Discription: print usage info.
 *	Arguments: none.
 *	Returns: none - exits tho.
 */

void
usage() {
  fprintf(stdout,
"Usage: xman ={ignored}x{ignored}+/-{x}+/-{y} [-r] [-d] [display:number]\n\n");
  fprintf(stdout,"The -r option turns on reverse video.\n");
  fprintf(stdout,"The -d option prints the default .Xdefaults to stdout.\n");
  exit(0);
}

/*	Function Name: printdefaults
 *	Discription: This functions prints out the default .Xdefaults to stdout
 *	Arguments: NONE 
 *	Returns: NONE - exits tho.
 */

void
printdefaults() {
  fprintf(stdout,"# These are the Default values used in for gnuplot\n");
  fprintf(stdout,"gnuplot.ReverseVideo:         off\n");
  fprintf(stdout,"gnuplot.Geometry:             =0x0+0+0\n");
  fprintf(stdout,"gnuplot.Font:                 fixed\n");
  exit(0);
}
  

/*	Function Name: get_geom
 *	Description: Gets the window geometery by XParseGeometry().
 *	Arguments: string - the string to parse.
 *	Returns: none.
 */

get_geom(string,xw_defs)
char * string;
Xdefs * xw_defs;
{

int x,y,width,height,bitmask;

/* get the geometery information, notice size ignored. */

  bitmask = XParseGeometry(string,&x,&y,&width,&height);
  if (!(bitmask & XValue))	/* if no x value given. */
    xw_defs->x = 0;		/* only used if of no wm given */
  else
    xw_defs->x = x;
  if (!(bitmask & YValue))	/* if no y value given. */
    xw_defs->y = 0;		/* only use if of no wm given */
  else
    xw_defs->y = y;
}

/*	Function Name: xcheck_args
 *	Description: This checks the command line arguments to print usage
 *                   info if incorrect ones area give or .xdefaults if asked
 *                   for.
 *	Arguments: number - the number of arguements.
 *                 strs   - the arguements themselves.
 *	Returns: none, may exit tho.
 */

xcheck_args(number,strs)
int number;			/* agrument count. */
char ** strs;			/* argument strings. */
{

  int i;

  for (i = 1; i < number; i++) {
    if (strs[i][0] == '-') {
      switch (strs[i][1]) {
	case 'r':			/* switch to reverse video. */
	  continue;		        /* this case okay. */
	case 'd':			/* print defaults to stdout. */
	  printdefaults();
	  break;
	case 'f':
	  i++;
	  continue;
	default:
	  usage();		/* print usage. */
	}
    }
    if (strs[i][0] == '=') {	/* this is geometery info, and okay too. */
      continue;
    }

    /* if we don't find something of the form foo:(single character) assume 
       it is a mistake and print usage info. */

    if (strs[i][strlen(strs[i])-2] != ':')
      usage();
  }
}
